{% extends "base.html" %}

{% block content %}
    {% include 'partials/_quiz_content.html' %}
{% endblock %}

{% block scripts %}
    <script src="{{ url_for('static', filename='js/vue/quiz.js') }}"></script>
{% endblock %}