{% extends "base.html" %}

{% block content %}

<!-- Overlay notifies user they're not logged in -->
<div id="overlay" class="overlay" style="display: flex;">
  <div class="overlay-content">
    <a href="{{ url_for('google_login.login') }}">Log in with your @rug.nl account</a>
  </div>
</div>

<div class="content-row">
    <!-- fake left-menu -->
    <div class="left-menu">
        <h3>Menu</h3>
    </div>
    <!-- next to menu, just show a status-line: not logged in: -->
    <div class="main-content">
        <div class="status-line">
            <p>Not logged in</p>
        </div>
    </div>
</div>
{% endblock %}

{% block scripts %}
{% endblock %}
