<div class="divider"></div>
<div class="footer-bar">
    v{{ version}}
    // Copyright 2024 - 2025 Sebastiaan Mathôt, Wouter Kruijne, <a href="https://rug.nl">University of Groningen</a>
    // <a href='https://github.com/rug-gmw/heymans'>Open source GPL3</a>
</div>
